#include <16f57.h>

#fuses RC,NOWDT,NOPROTECT
#use delay(clock=100000)
#define SLEEP_Time	10

unsigned char PORTA, PORTB, PORTC,TMR0;


#locate	PORTA = 0X05
#locate	PORTB = 0X06
#locate PORTC = 0X07
#locate TMR0 = 0X01




int16 LFSR = 1;
int1 Output_bit;
unsigned char 	DIE_OUT;
unsigned char Animate;
int Tumble;
unsigned char DIE_Flag;
unsigned char Time_2_SLEEP;


//Define Dice output to light appropriate LEDs

unsigned char DIE[6] = {0x08,0x41,0x49,0x55,0x5D,0x77};

#bit	ROLL1 = PORTA.1
#bit	ROLL2 = PORTA.0
#bit	BUZZER = PORTA.2

#bit	TOF = TMR0.7


#bit LFSR_16 = LFSR.15
#bit LFSR_5 = LFSR.4
#bit LFSR_4 = LFSR.3
#bit LFSR_3 = LFSR.2
#bit LFSR_2 = LFSR.1
#bit LFSR_1 = LFSR.0


//Function Prototypes
void Initialize (void);
void Get_Inputs (void);
void Output_DIE(void);
void randnum_generator(void);
void Check_button(void);

void Initialize (void)
{
	//In order to initialize our PORT directions
	//in-line Assembly is required in order to 
	//access the TRIS(PORT direction)register.
	
	//Set the PORT directions as follows:
	//	PORTB and PORTC pins are connected to the 
	//	left and right dice LEDS repectively. 
	//	Therefore, set both these registers as output.
	
	//	PORTA is used to take inputs from the ROLL1 and 
	//	ROLL2 pushbuttons on RA2 and RA1 respectively.
	//	RA3 will be used as the output to our buzzer.
	//	Therefore, set RA1/RA2 as inputs and RA3 as output
	
	//Initialize the PORTS to a known state (i.e. 0)
		PORTA = 0;
		PORTB = 0;
		PORTC = 0;
	
	#asm	//start a block of assembly code
		
		
		movlw	0x03	//Setup PORTA for RA1 and RA2 as inputs
						//and the rest as outputs by moving 0x03
						//into the working register (clearing a pin's
						//direction to '0' makes it an output, setting
						//a pin's direction to '1' makes it an input
						
						//then
						
		tris	PORTA	//issuing the tris command moves the 
						//contents of the working register into
						//the PORTA direction register
						
		movlw	0X00	//PORTB and PORTC pins are used to output to the
						//LEDS. Therefore, clear all bits in the PORTB and
						//PORTC direction registers
						
						
		tris	PORTB
		tris	PORTC
		
		movlw	0x07
		
		OPTION
		
	#endasm	//end a block of Assembly code
	

	PORTB = 0XFF;
	PORTC = 0XFF;
	TMR0 = 0;
	Time_2_SLEEP = SLEEP_Time;

}
	
		
			



void main(void)

{	
	

	Initialize();
	
 while(1)
 {
	 delay_ms(100); 	
	 randnum_generator(); //generate a random number
	 Check_button(); //Check for a button press

	 if(Time_2_SLEEP==0) 
	 {	 
		 
		 PORTB = 0;
	     PORTC = 0;
		 SLEEP();
	}

	//if Time_2_SLEEP times out go to SLEEP
	

	
	else
		{	while(TOF);
			if(!TMR0) 
			{
				--Time_2_SLEEP;

			}
	
		}


	}
}


void randnum_generator(void)
{
	
			Output_bit = LFSR_16;
			LFSR <<=1;
			
			//XOR the LFSR register with the characteristic polynomial
			//X^16 + X^5 + X^3 + X^2 + 1		
		   LFSR_1 = Output_bit ^ LFSR_16 ^LFSR_5 ^LFSR_3^LFSR_2;
		 
		   if((LFSR & 0X00FF) == 0) LFSR = 1; //if the LFSR is zero reload with 1
		   
		   
}


void Check_button(void)
{
	

	if(ROLL1 == 0)
	{
		delay_ms(2); //Delay for 2ms to debounce the switch
		
		//while the ROLL1 button is pressed decrement the current
		//LFSR value
		Animate = 6;
		while(ROLL1 == 0) 
			{
				
			++LFSR;
			delay_ms(50);
			--Animate;
			if(Animate > 5)Animate = 5;
			PORTB = DIE[Animate];
			BUZZER ^= 1;			
			
		   }
		 Tumble = 10;
		 while(Tumble <= 40)
		 {
		
			delay_ms(Tumble);
			--Animate; 
			if(Animate > 5)Animate = 5;
			PORTB = DIE[Tumble%5];
			BUZZER ^= 1;
			++Tumble;	
		 }
		
		DIE_Flag = 1;
		Time_2_SLEEP = SLEEP_Time;
		Output_DIE(); //output the result to the DIE LEDs
			
	}
	

	if(ROLL2 == 0)
	{
		delay_ms(2); //Delay for 2ms to debounce the switch
		
		//while the ROLL1 button is pressed decrement the current
		//LFSR value
		Animate = 6;
		while(ROLL2 == 0) 
			{
				
			++LFSR;
			delay_ms(50);
			--Animate;
			if(Animate > 5)Animate = 5;
			PORTC = DIE[Animate];
			BUZZER ^= 1;			
			
		   }
		 Tumble = 10;
		 while(Tumble <= 40)
		 {
			 ++LFSR;
			delay_ms(Tumble);
			--Animate;
			if(Animate > 5)Animate = 5;
			PORTC = DIE[Animate];
			BUZZER ^= 1;
			++Tumble;	
		 }
		
		DIE_Flag = 0;
		Time_2_SLEEP = SLEEP_Time;
		Output_DIE(); //output the result to the DIE LEDs
			
	}
		
	
}

void Output_DIE(void)
{	   
		   //Moding a number by a value will return a remainder no greater
		   //than that value  
		   DIE_OUT = (LFSR % 6);
	if(DIE_Flag) PORTB = DIE[DIE_OUT];
	if(!DIE_Flag) PORTC = DIE[DIE_OUT];
		   	
}
		   
			   
		   
		